import QtQuick 2.0
import Sailfish.Silica 1.0

Page {
    id: page

    // The effective value will be restricted by ApplicationWindow.allowedOrientations
    allowedOrientations: Orientation.All

    onStatusChanged: {
        if (status === PageStatus.Active) {
            if (login.isLoggedIn()) {
                pageStack.clear()
                pageStack.replace(Qt.resolvedUrl("MainView.qml"));
            }
        }
    }

    // To enable PullDownMenu, place our content in a SilicaFlickable
    SilicaFlickable {
        anchors.fill: parent

        // PullDownMenu and PushUpMenu must be declared in SilicaFlickable, SilicaListView or SilicaGridView
        PullDownMenu {
            MenuItem {
                text: qsTr("About")
                onClicked: pageStack.push(Qt.resolvedUrl("AboutPage.qml"))
            }
            MenuItem {
                text: qsTr("Register")
                onClicked: pageStack.push(Qt.resolvedUrl("AboutPage.qml"))
            }
        }


        Connections {
            target: login
            onLoginSuccess: {
                pageStack.clear()
                pageStack.replace(Qt.resolvedUrl("MainView.qml"));
            }
        }

        // Tell SilicaFlickable the height of its content.
        contentHeight: column.height

        // Place our content in a Column.  The PageHeader is always placed at the top
        // of the page, followed by our content.
        Column {
            id: column

            width: page.width
            spacing: Theme.paddingLarge

            function validateLogin() {
                if (!user_parse_error.visible && password.length > 0
                        && mxid.length > 0)
                    pageStack.pushAttached(loggingIn)
                else if (page.forwardNavigation)
                    pageStack.popAttached(undefined, PageStackAction.Animated)
            }

            PageHeader {
                title: qsTr("Login")
            }

            TextArea {
                id: user_parse_error
                visible: false
                color: "red"
                font.pixelSize: 0
                wrapMode: Text.WrapAtWordBoundaryOrAnywhere
                readOnly: true
                anchors {
                    left: parent.left
                    right: parent.right
                }

                Connections {
                    target: login
                    onUserParseError: {
                        user_parse_error.text = error
                        user_parse_error.visible = true
                    }
                }
                Connections {
                    target: login
                    onHomeserverInvalid: {
                        user_parse_error.text = error
                        user_parse_error.visible = true
                        column.validateLogin()
                    }
                }
                Connections {
                    target: login
                    onLoginError: {
                        user_parse_error.text = error
                        user_parse_error.visible = true
                        pageStack.navigateBack()
                        column.validateLogin()
                    }
                }
            }

            TextField {
                id: mxid
                anchors {
                    left: parent.left
                    right: parent.right
                }

                //: TextField hint for entering matrix id
                label: qsTr("@matrix-id:your.homeserver")
                placeholderText: label
                EnterKey.enabled: text || inputMethodComposing
                EnterKey.iconSource: "image://theme/icon-m-enter-next"
                EnterKey.onClicked: {
                    if (homeserver.text.length == 0) {
                        homeserver.visible = false
                    }
                    login.mxidEntered(text)
                    password.focus = true
                    column.validateLogin()
                }

                validator: RegExpValidator {
                    regExp: /@[a-z0-9\._=\-/]+:.+/
                }

                inputMethodHints: Qt.ImhNoPredictiveText
            }

            PasswordField {
                id: password
                anchors {
                    left: parent.left
                    right: parent.right
                }

                //: Simply the password prompt
                label: qsTr("Password")
                placeholderText: label
                EnterKey.enabled: text || inputMethodComposing
                EnterKey.iconSource: "image://theme/icon-m-enter-next"
                EnterKey.onClicked: {
                    if (homeserver.visible)
                        homeserver.focus = true

                    column.validateLogin()
                }
            }

            TextField {
                id: homeserver
                anchors {
                    left: parent.left
                    right: parent.right
                }
                //: TextField hint for entering homeserver
                label: qsTr("your.homeserver:optional port")
                placeholderText: label
                EnterKey.enabled: text || inputMethodComposing
                EnterKey.iconSource: "image://theme/icon-m-enter-next"
                EnterKey.onClicked: {
                    user_parse_error.visible = false
                    login.homeserverEntered(text)
                    column.validateLogin()
                }
                validator: RegExpValidator {
                    regExp: /.+/
                }
                visible: false
                Connections {
                    target: login
                    onHomeserverInvalid: {
                        homeserver.text = server
                        homeserver.visible = true
                    }
                }

                //inputMethodHints: Qt.ImhNoPredictiveText
            }
        }
    }

    Component {
        id: loggingIn
        Page {

            BusyIndicator {
                size: BusyIndicatorSize.Large
                anchors.centerIn: parent
                running: true
            }

            onStatusChanged: {
                if (status === PageStatus.Activating) {
                    console.log("Login call from qml")
                    login.login(mxid.text, password.text, homeserver.text)
                }
            }
        }
    }
}
