import QtQuick 2.0
import Sailfish.Silica 1.0

Page {
    id: mainPage

    // The effective value will be restricted by ApplicationWindow.allowedOrientations
    allowedOrientations: Orientation.All

    Component {
        id: roomContextMenu
        ContextMenu {
            MenuItem {
                text: "Remove"
                onClicked: highlight()
            }
        }
    }

    Connections {
        target: login
        onLoggedOut: pageStack.replace(Qt.resolvedUrl("LoginPage.qml"))
    }

    SilicaListView {
        id: roomView
        anchors.fill: parent

        ScrollDecorator { flickable: roomView }

        // PullDownMenu and PushUpMenu must be declared in SilicaFlickable, SilicaListView or SilicaGridView
        PullDownMenu {
            MenuItem {
                text: qsTr("About")
                onClicked: pageStack.push(Qt.resolvedUrl("AboutPage.qml"))
            }
            MenuItem {
                text: qsTr("Exit")
                onClicked: Qt.quit()
            }
            MenuItem {
                text: qsTr("Logout")
                onClicked: login.logout()
            }
        }

        header: PageHeader {
            title: qsTr("Rooms")
        }

        model: rooms

        delegate: ListItem {
            id: roomItem
            menu: roomContextMenu

            onClicked: pageStack.push(Qt.resolvedUrl("ChatPage.qml"), {room: model.roomModel} );

            contentHeight: mainRow.height + Theme.paddingSmall
            contentWidth: parent.width

            Row {
                anchors.left: parent.left
                anchors.right: parent.right
                spacing: Theme.paddingMedium
                width: roomItem.width

                id: mainRow

                Label {
                    BackgroundItem {
                        Rectangle {
                            color: Theme.highlightBackgroundColor
                        }
                    }

                    color: unreadNotifications > 0 ? Theme.highlightColor : Theme.secondaryColor
                    width: 2 * Theme.fontSizeLarge
                    id: unreadMessages
                    text: unreadNotifications
                    font.pixelSize: unreadNotifications < 100 ? Theme.fontSizeHuge : Theme.fontSizeLarge
                    horizontalAlignment: Text.AlignRight
                }

                Column {
                    width: mainRow.width - unreadMessages.width
                    - roomImage.width - 3* Theme.paddingMedium
                    Label {
                        anchors {
                            left: parent.left
                            right: parent.right
                        }

                        id: roomName
                        text: model.roomName
                        truncationMode: TruncationMode.Fade
                        color: Theme.primaryColor
                        font.pixelSize: Theme.fontSizeLarge
                    }

                    Label {
                        anchors {
                            left: parent.left
                            right: parent.right
                        }

                        id: lastMessage
                        text: model.lastMessage
                        color: unreadNotifications > 0 ? Theme.primaryColor : Theme.secondaryColor
                        wrapMode: Text.Wrap
                        truncationMode: TruncationMode.Fade
                        maximumLineCount: 3
                        font.pixelSize: Theme.fontSizeExtraSmall
                    }
                }

                Image {
                    id: roomImage
                    width: unreadMessages.width
                    height: unreadMessages.height
                    sourceSize.width: unreadMessages.width
                    sourceSize.height: unreadMessages.height
                    fillMode: Image.PreserveAspectFit
                    source: model.roomPicture
                    asynchronous: true
                }
            }
        }
    }
}
