#include "dbusactivation.h"

#include <QDebug>

#include "spoonadaptor.h"

static const QString c_dbusServiceName = QStringLiteral("dev.neko.spoon");
static const QString c_dbusObjectPath = QStringLiteral("/dev/neko/spoon");
static const QString c_dbusInterface = QStringLiteral("dev.neko.spoon");
static const QString c_dbusMethod = QStringLiteral("showApp");

DBusActivator::DBusActivator(QGuiApplication *parent) : app(parent)
{
    QDBusConnection dbus = QDBusConnection::sessionBus();
    new SpoonAdaptor(this);
    qDebug() << "Register object" << dbus.registerObject(c_dbusObjectPath, this); //object path
    if (!isRegistered()) {
        bool ready = dbus.registerService(c_dbusServiceName);
        qDebug() << "Register service" << ready;
        if (!ready) {
            qWarning() << "Service already registered, exiting...";
            app->quit();
        }
    }
}
DBusActivator::~DBusActivator()
{
    QDBusConnection dbus = QDBusConnection::sessionBus();

    dbus.unregisterObject(c_dbusObjectPath); //object path
    bool ready = dbus.unregisterService(c_dbusServiceName);
    qDebug() << "Unregister service" << ready; //object path

}

bool DBusActivator::isRegistered()
{
    return QDBusConnection::sessionBus().interface()->isServiceRegistered(c_dbusServiceName);
}

bool DBusActivator::raiseApp()
{
    QDBusConnection dbus = QDBusConnection::sessionBus();
    QDBusMessage showUi = QDBusMessage::createMethodCall(
                              c_dbusServiceName,
                              c_dbusObjectPath,
                              c_dbusInterface,
                              c_dbusMethod);

    auto result = dbus.call(showUi);
    if (result.errorName() != "")
        qDebug() << result.errorName() << result.errorMessage();
    return true;
}

void DBusActivator::showApp() {
    if (!view) {
        qDebug() << "Construct view";

        view = spoon::showApp(app);
        connect(view, &QQuickView::destroyed, this, &DBusActivator::onViewDestroyed);
        // connect(view, &QQuickView::closing, this, &DBusActivator::onViewClosing); <- see
        // https://bugreports.qt.io/browse/QTBUG-36453
        connect(view, SIGNAL(closing(QQuickCloseEvent *)), this, SLOT(onViewClosing(QQuickCloseEvent *)));
    } else if (view->windowState() == Qt::WindowNoState) {
        view->create();
        view->showFullScreen();
    } else {
        view->raise();
        view->requestActivate();
    }
}

void DBusActivator::openRoom(const QString &roomId) {}

void DBusActivator::onViewDestroyed()
{
    view = nullptr;
}

void DBusActivator::onViewClosing(QQuickCloseEvent *v)
{
    Q_UNUSED(v)
    view->destroy();
    view->deleteLater();
}
