#ifndef ROOMMODEL_H
#define ROOMMODEL_H

#include <QAbstractListModel>
#include <QSharedPointer>
#include <QSortFilterProxyModel>

#include <map>
#include <string>
#include <vector>

#include "models/roommodel.h"

class RoomListSortProxy : public QSortFilterProxyModel {
    Q_OBJECT

  public:
    explicit RoomListSortProxy(QObject *parent = 0) : QSortFilterProxyModel(parent) {}

  public slots:
    void hideTag(QString tag);
    void showTag(QString tag);
    bool isHidden(QString tag) const {
        auto i = hiddenTags.find(tag);
        return i != hiddenTags.end() && i->second;
    }

  protected:
    bool lessThan(const QModelIndex &left, const QModelIndex &right) const override;
    bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const override;

    std::map<QString, bool> hiddenTags;
};

class RoomListModel : public QAbstractListModel {
    Q_OBJECT
  public:
    enum RoomRoles {
        RoomModelRole,
        RoomNameRole,
        UnreadNotificationsRole,
        LastMessageRole,
        RoomPictureRole,
        TagRole,
        IsInvite,
    };

    QHash<int, QByteArray> roleNames() const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;

    QSharedPointer<Room> room(std::string id) { return roomById[id]; }

    explicit RoomListModel(QObject *parent = 0);

  public slots:
    void addRoom(QSharedPointer<Room> room);
    void updateRoom(QSharedPointer<Room> room);
    void clear();

  private:
    QModelIndex calcIndex(std::string id);
    std::vector<QSharedPointer<Room>> rooms;
    std::map<std::string, QSharedPointer<Room>> roomById;
};

#endif // ROOMMODEL_H
